@@ -198,103 +198,161 @@ PKNCA.set.summary("clast.obs", business.geomean, business.geocv)
 #' @export
 pk.calc.thalf.eff <- function(mrt)
   log(2)*mrt
-pk.calc.thalf.eff.obs <- function(mrt.obs)
-  pk.calc.thalf.eff(mrt.obs)
-pk.calc.thalf.eff.pred <- function(mrt.pred)
-  pk.calc.thalf.eff(mrt.pred)
 ## Add the columns to the interval specification
 add.interval.col("thalf.eff.obs",
-                 FUN="pk.calc.thalf.eff.obs",
+                 FUN="pk.calc.thalf.eff",
                  values=c(FALSE, TRUE),
                  desc="The effective half-life (as determined from the MRTobs)",
+                 formalsmap=list(mrt="mrt.obs"),
                  depends=c("mrt.obs"))
 PKNCA.set.summary("thalf.eff.obs", business.geomean, business.geocv)
 add.interval.col("thalf.eff.pred",
-                 FUN="pk.calc.thalf.eff.pred",
+                 FUN="pk.calc.thalf.eff",
                  values=c(FALSE, TRUE),
                  desc="The effective half-life (as determined from the MRTpred)",
+                 formalsmap=list(mrt="mrt.pred"),
                  depends=c("mrt.pred"))
 PKNCA.set.summary("thalf.eff.pred", business.geomean, business.geocv)
+add.interval.col("thalf.eff.last",
+                 FUN="pk.calc.thalf.eff",
+                 values=c(FALSE, TRUE),
+                 desc="The effective half-life (as determined from the MRTlast)",
+                 formalsmap=list(mrt="mrt.last"),
+                 depends=c("mrt.last"))
+PKNCA.set.summary("thalf.eff.last", business.geomean, business.geocv)
+add.interval.col("thalf.eff.iv.obs",
+                 FUN="pk.calc.thalf.eff",
+                 values=c(FALSE, TRUE),
+                 desc="The effective half-life (as determined from the intravenous MRTobs)",
+                 formalsmap=list(mrt="mrt.iv.obs"),
+                 depends=c("mrt.iv.obs"))
+PKNCA.set.summary("thalf.eff.iv.obs", business.geomean, business.geocv)
+add.interval.col("thalf.eff.iv.pred",
+                 FUN="pk.calc.thalf.eff",
+                 values=c(FALSE, TRUE),
+                 desc="The effective half-life (as determined from the intravenous MRTpred)",
+                 formalsmap=list(mrt="mrt.iv.pred"),
+                 depends=c("mrt.iv.pred"))
+PKNCA.set.summary("thalf.eff.iv.pred", business.geomean, business.geocv)
+add.interval.col("thalf.eff.iv.last",
+                 FUN="pk.calc.thalf.eff",
+                 values=c(FALSE, TRUE),
+                 desc="The effective half-life (as determined from the intravenous MRTlast)",
+                 formalsmap=list(mrt="mrt.iv.last"),
+                 depends=c("mrt.iv.last"))
+PKNCA.set.summary("thalf.eff.iv.last", business.geomean, business.geocv)
 
 #' Calculate the AUC percent extrapolated
 #' 
-#' @param auclast the area under the curve from time 0 to the last measurement
-#'   above the limit of quantification
-#' @param aucinf,aucinf.obs,aucinf.pred the area under the curve from time 0 to
-#'   infinity
-#' @return the numeric value of the AUC percent extrapolated
+#' @param auclast the area under the curve from time 0 to the last 
+#'   measurement above the limit of quantification
+#' @param aucinf the area under the curve from time 0 to infinity
+#' @return The numeric value of the AUC percent extrapolated or 
+#'   \code{NA_real_} if andy of the following are true 
+#'   \code{is.na(aucinf)}, \code{is.na(auclast)}, \code{aucinf <= 0},
+#'   or \code{auclast <= 0}.
 #' @export
 pk.calc.aucpext <- function(auclast, aucinf) {
-  if (auclast >= aucinf)
-    warning("auclast should be less than aucinf")
-  100*(1-auclast/aucinf)
+  scalar_auclast <- length(auclast) == 1
+  scalar_aucinf <- length(aucinf) == 1
+  if (scalar_auclast | scalar_aucinf) {
+    # no length checking needs to occur
+  } else if ((!scalar_auclast & !scalar_aucinf) &
+             length(auclast) != length(aucinf)) {
+    stop("auclast and aucinf must either be a scalar or the same length.")
+  }
+  ret <- rep(NA_real_, max(c(length(auclast), length(aucinf))))
+  mask_na <-
+    is.na(auclast) |
+    is.na(aucinf)
+  mask_negative <-
+    !mask_na &
+    (aucinf <= 0 |
+       auclast <= 0)
+  mask_greater <-
+    !mask_na &
+    (auclast >= aucinf)
+  mask_calc <- !mask_na
+  if (any(mask_greater))
+    warning("aucpext is typically only calculated when aucinf is greater than auclast.")
+  if (any(mask_negative))
+    warning("aucpext is typically only calculated when both aucinf and auclast are positive.")
+  ret[mask_calc] <-
+    100*(1-auclast[mask_calc]/aucinf[mask_calc])
+  ret
 }
 
-#' @describeIn pk.calc.aucpext Compute the percent extrapolated AUCinf from the 
-#'   observed Clast
-#' @export
-pk.calc.aucpext.obs <- function(auclast, aucinf.obs)
-  pk.calc.aucpext(auclast, aucinf.obs)
-#' @describeIn pk.calc.aucpext Compute the percent extrapolated AUCinf from the 
-#'   observed Clast
-#' @export
-pk.calc.aucpext.pred <- function(auclast, aucinf.pred)
-  pk.calc.aucpext(auclast, aucinf.pred)
-
 ## Add the columns to the interval specification
 add.interval.col("aucpext.obs",
-                 FUN="pk.calc.aucpext.obs",
+                 FUN="pk.calc.aucpext",
                  values=c(FALSE, TRUE),
                  desc="Percent of the AUCinf that is extrapolated after Tlast calculated from the observed Clast",
+                 formalsmap=list(aucinf="aucinf.obs"),
                  depends=c("auclast", "aucinf.obs"))
 PKNCA.set.summary("aucpext.obs", business.mean, business.sd)
 add.interval.col("aucpext.pred",
-                 FUN="pk.calc.aucpext.pred",
+                 FUN="pk.calc.aucpext",
                  values=c(FALSE, TRUE),
                  desc="Percent of the AUCinf that is extrapolated after Tlast calculated from the predicted Clast",
+                 formalsmap=list(aucinf="aucinf.pred"),
                  depends=c("auclast", "aucinf.pred"))
 PKNCA.set.summary("aucpext.pred", business.mean, business.sd)
 
 #' Calculate the elimination rate (Kel)
 #'
-#' @param mrt,mrt.obs,mrt.pred the mean residence time
+#' @param mrt the mean residence time
 #' @return the numeric value of the elimination rate
 #' @export
 pk.calc.kel <- function(mrt)
   1/mrt
-#' @describeIn pk.calc.kel Calculate Kel with observed Clast
-#' @export
-pk.calc.kel.obs <- function(mrt.obs)
-  pk.calc.kel(mrt.obs)
-#' @describeIn pk.calc.kel Calculate Kel with predicted Clast
-#' @export
-pk.calc.kel.pred <- function(mrt.pred)
-  pk.calc.kel(mrt.pred)
 ## Add the columns to the interval specification
 add.interval.col("kel.obs",
-                 FUN="pk.calc.kel.obs",
+                 FUN="pk.calc.kel",
                  values=c(FALSE, TRUE),
                  desc="Elimination rate (as calculated from the MRT with observed Clast)",
+                 formalsmap=list(mrt="mrt.obs"),
                  depends=c("mrt.obs"))
 PKNCA.set.summary("kel.obs", business.geomean, business.geocv)
 add.interval.col("kel.pred",
-                 FUN="pk.calc.kel.pred",
+                 FUN="pk.calc.kel",
                  values=c(FALSE, TRUE),
                  desc="Elimination rate (as calculated from the MRT with predicted Clast)",
+                 formalsmap=list(mrt="mrt.pred"),
                  depends=c("mrt.pred"))
 PKNCA.set.summary("kel.pred", business.geomean, business.geocv)
+add.interval.col("kel.last",
+                 FUN="pk.calc.kel",
+                 values=c(FALSE, TRUE),
+                 desc="Elimination rate (as calculated from the MRT using AUClast)",
+                 formalsmap=list(mrt="mrt.last"),
+                 depends=c("mrt.last"))
+PKNCA.set.summary("kel.last", business.geomean, business.geocv)
+add.interval.col("kel.iv.obs",
+                 FUN="pk.calc.kel",
+                 values=c(FALSE, TRUE),
+                 desc="Elimination rate (as calculated from the intravenous MRTobs)",
+                 formalsmap=list(mrt="mrt.iv.obs"),
+                 depends=c("mrt.iv.obs"))
+PKNCA.set.summary("kel.iv.obs", business.geomean, business.geocv)
+add.interval.col("kel.iv.pred",
+                 FUN="pk.calc.kel",
+                 values=c(FALSE, TRUE),
+                 desc="Elimination rate (as calculated from the intravenous MRTpred)",
+                 formalsmap=list(mrt="mrt.iv.pred"),
+                 depends=c("mrt.iv.pred"))
+PKNCA.set.summary("kel.iv.pred", business.geomean, business.geocv)
+add.interval.col("kel.iv.last",
+                 FUN="pk.calc.kel",
+                 values=c(FALSE, TRUE),
+                 desc="Elimination rate (as calculated from the intravenous MRTlast)",
+                 formalsmap=list(mrt="mrt.iv.last"),
+                 depends=c("mrt.iv.last"))
+PKNCA.set.summary("kel.iv.last", business.geomean, business.geocv)
 
 #' Calculate the (observed oral) clearance
 #' 
 #' @param dose the dose administered
-#' @param aucall The area under the concentration-time curve from 0 to the last 
-#'   measurement above the limit of quantifiation (LOQ) plus the triangle to the
-#'   first concentration below the LOQ.
-#' @param auclast The area under the concentration-time curve from 0 to the last
-#'   measurement above the LOQ.
-#' @param aucinf,aucinf.obs,aucinf.pred The area under the concentration-time 
-#'   curve from 0 to infinity (the next dose on a regular schedule at 
-#'   steady-state)
+#' @param auc The area under the concentration-time curve.
 #' @return the numeric value of the total (CL) or observed oral clearance (CL/F)
 #' @details If \code{dose} is the same length as the other inputs, then the 
 #'   output will be the same length as all of the inputs; the function assumes 
@@ -306,53 +364,45 @@ PKNCA.set.summary("kel.pred", business.geomean, business.geocv)
 #'   Pharmacokinetic & Pharmacodynamic Data Analysis: Concepts and Applications,
 #'   4th Edition.  Stockholm, Sweden: Swedish Pharmaceutical Press, 2000. 86-7.
 #' @export
-pk.calc.cl <- function(dose, aucinf) {
-  if (length(aucinf) == 1)
+pk.calc.cl <- function(dose, auc) {
+  if (length(auc) == 1) {
     dose <- sum(dose)
-  dose/aucinf
+  }
+  ret <- dose/auc
+  mask_zero <- !is.na(auc) & (auc <= 0)
+  if (any(mask_zero)) {
+    ret[mask_zero] <- NA_real_
+  }
+  ret
 }
-#' @describeIn pk.calc.cl Compute the clearance from AUClast
-#' @export
-pk.calc.cl.last <- function(dose, auclast)
-  pk.calc.cl(dose, auclast)
-#' @describeIn pk.calc.cl Compute the clearance from AUCall
-#' @export
-pk.calc.cl.all <- function(dose, aucall)
-  pk.calc.cl(dose, aucall)
-#' @describeIn pk.calc.cl Compute the clearance from AUCinf (calculated from 
-#'   observed Clast)
-#' @export
-pk.calc.cl.obs <- function(dose, aucinf.obs)
-  pk.calc.cl(dose, aucinf.obs)
-#' @describeIn pk.calc.cl Compute the clearance from AUCinf (calculated from
-#'   predicted Clast)
-#' @export
-pk.calc.cl.pred <- function(dose, aucinf.pred)
-  pk.calc.cl(dose, aucinf.pred)
 
 ## Add the columns to the interval specification
 add.interval.col("cl.last",
-                 FUN="pk.calc.cl.last",
+                 FUN="pk.calc.cl",
                  values=c(FALSE, TRUE),
                  desc="Clearance or observed oral clearance calculated to Clast",
+                 formalsmap=list(auc="auclast"),
                  depends=c("auclast"))
 PKNCA.set.summary("cl.last", business.geomean, business.geocv)
 add.interval.col("cl.all",
-                 FUN="pk.calc.cl.all",
+                 FUN="pk.calc.cl",
                  values=c(FALSE, TRUE),
                  desc="Clearance or observed oral clearance calculated with AUCall",
+                 formalsmap=list(auc="aucall"),
                  depends=c("aucall"))
 PKNCA.set.summary("cl.all", business.geomean, business.geocv)
 add.interval.col("cl.obs",
-                 FUN="pk.calc.cl.obs",
+                 FUN="pk.calc.cl",
                  values=c(FALSE, TRUE),
                  desc="Clearance or observed oral clearance calculated with observed Clast",
+                 formalsmap=list(auc="aucinf.obs"),
                  depends=c("aucinf.obs"))
 PKNCA.set.summary("cl.obs", business.geomean, business.geocv)
 add.interval.col("cl.pred",
-                 FUN="pk.calc.cl.pred",
+                 FUN="pk.calc.cl",
                  values=c(FALSE, TRUE),
                  desc="Clearance or observed oral clearance calculated with predicted Clast",
+                 formalsmap=list(auc="aucinf.pred"),
                  depends=list("aucinf.pred"))
 PKNCA.set.summary("cl.pred", business.geomean, business.geocv)
 
@@ -365,8 +415,17 @@ PKNCA.set.summary("cl.pred", business.geomean, business.geocv)
 #' @param auc2 The AUC from 0 to infinity or 0 to tau administered in
 #'   route or method 2
 #' @export
-pk.calc.f <- function(dose1, auc1, dose2, auc2)
-  (auc2/dose2)/(auc1/dose1)
+pk.calc.f <- function(dose1, auc1, dose2, auc2) {
+  ret <- (auc2/dose2)/(auc1/dose1)
+  mask_zero <-
+    is.na(auc1)  | (auc1 <= 0) |
+    is.na(dose2) | (dose2 <= 0) |
+    is.na(dose1) | (dose1 <= 0)
+  if (any(mask_zero)) {
+    ret[mask_zero] <- NA_real_
+  }
+  ret
+}
 add.interval.col("f",
                  FUN="pk.calc.f",
                  values=c(FALSE, TRUE),
@@ -374,60 +433,116 @@ add.interval.col("f",
                  depends=c())
 PKNCA.set.summary("f", business.geomean, business.geocv)
 
-#' Calcuate the mean residence time (MRT)
-#'
-#' @param aucinf,aucinf.obs,aucinf.pred the AUC from 0 to infinity or
-#'   0 to tau at steady-state
-#' @param auclast the AUC from 0 to the last concentration above the
-#'   limit of quantification (LOQ)
-#' @param aumcinf,aumcinf.obs,aumcinf.pred the AUMC from 0 to infinity
-#'   or 0 to tau at steady-state
-#' @param aumclast the AUMC from 0 to the last concentration above the
-#'   LOQ
+#' Calculate the mean residence time (MRT) for single-dose data or 
+#' linear multiple-dose data.
+#' 
+#' @param auc the AUC from 0 to infinity or 0 to tau
+#' @param aumc the AUMC from 0 to infinity or 0 to tau
+#' @param duration.dose The duration of the dose (usually an infusion
+#'   duration for an IV infusion)
 #' @return the numeric value of the mean residence time
+#' @seealso \code{\link{pk.calc.mrt.md}}
 #' @export
-pk.calc.mrt <- function(aucinf, aumcinf)
-  aumcinf/aucinf
-#' @describeIn pk.calc.mrt Calcuate the mean residence time (MRT)
-#'   using observed Clast
-#' @export
-pk.calc.mrt.obs <- function(aucinf.obs, aumcinf.obs)
-  pk.calc.mrt(aucinf.obs, aumcinf.obs)
-#' @describeIn pk.calc.mrt Calcuate the mean residence time (MRT)
-#'   using predicted Clast
-#' @export
-pk.calc.mrt.pred <- function(aucinf.pred, aumcinf.pred)
-  pk.calc.mrt(aucinf.pred, aumcinf.pred)
+pk.calc.mrt <- function(auc, aumc) {
+  pk.calc.mrt.iv(auc, aumc, duration.dose=0)
+}
 ## Add the columns to the interval specification
 add.interval.col("mrt.obs",
-                 FUN="pk.calc.mrt.obs",
+                 FUN="pk.calc.mrt",
                  values=c(FALSE, TRUE),
                  desc="The mean residence time to infinity using observed Clast",
+                 formalsmap=list(auc="aucinf.obs", aumc="aumcinf.obs"),
                  depends=c("aucinf.obs", "aumcinf.obs"))
 PKNCA.set.summary("mrt.obs", business.geomean, business.geocv)
 add.interval.col("mrt.pred",
-                 FUN="pk.calc.mrt.pred",
+                 FUN="pk.calc.mrt",
                  values=c(FALSE, TRUE),
                  desc="The mean residence time to infinity using predicted Clast",
+                 formalsmap=list(auc="aucinf.pred", aumc="aumcinf.pred"),
                  depends=c("aucinf.pred", "aumcinf.pred"))
 PKNCA.set.summary("mrt.pred", business.geomean, business.geocv)
-
-#' @describeIn pk.calc.mrt Calculate the mean residence time (MRT) to the last
-#'   concentration above the limit of quantification
-#' @export
-pk.calc.mrt.last <- function(auclast, aumclast)
-  pk.calc.mrt(auclast, aumclast)
-## Add the column to the interval specification
 add.interval.col("mrt.last",
-                 FUN="pk.calc.mrt.last",
+                 FUN="pk.calc.mrt",
                  values=c(FALSE, TRUE),
                  desc="The mean residence time to the last observed concentration above the LOQ",
+                 formalsmap=list(auc="auclast", aumc="aumclast"),
                  depends=list("auclast", "aumclast"))
 PKNCA.set.summary("mrt.last", business.geomean, business.geocv)
 
+#' @describeIn pk.calc.mrt MRT for an IV infusion
+#' @export
+pk.calc.mrt.iv <- function(auc, aumc, duration.dose) {
+  ret <- aumc/auc - duration.dose/2
+  mask_zero <- is.na(auc) | auc <= 0
+  if (any(mask_zero)) {
+    ret[mask_zero] <- NA_real_
+  }
+  ret
+}
+## Add the columns to the interval specification
+add.interval.col("mrt.iv.obs",
+                 FUN="pk.calc.mrt.iv",
+                 values=c(FALSE, TRUE),
+                 desc="The mean residence time to infinity using observed Clast correcting for dosing duration",
+                 formalsmap=list(auc="aucinf.obs", aumc="aumcinf.obs"),
+                 depends=c("aucinf.obs", "aumcinf.obs"))
+PKNCA.set.summary("mrt.iv.obs", business.geomean, business.geocv)
+add.interval.col("mrt.iv.pred",
+                 FUN="pk.calc.mrt.iv",
+                 values=c(FALSE, TRUE),
+                 desc="The mean residence time to infinity using predicted Clast correcting for dosing duration",
+                 formalsmap=list(auc="aucinf.pred", aumc="aumcinf.pred"),
+                 depends=c("aucinf.pred", "aumcinf.pred"))
+PKNCA.set.summary("mrt.iv.pred", business.geomean, business.geocv)
+add.interval.col("mrt.iv.last",
+                 FUN="pk.calc.mrt.iv",
+                 values=c(FALSE, TRUE),
+                 desc="The mean residence time to the last observed concentration above the LOQ correcting for dosing duration",
+                 formalsmap=list(auc="auclast", aumc="aumclast"),
+                 depends=list("auclast", "aumclast"))
+PKNCA.set.summary("mrt.iv.last", business.geomean, business.geocv)
+
+#' Calculate the mean residence time (MRT) for multiple-dose data with
+#' nonlinear kinetics.
+#' 
+#' @param auctau the AUC from time 0 to the end of the dosing interval 
+#'   (tau).
+#' @param aumctau the AUMC from time 0 to the end of the dosing interval
+#'   (tau).
+#' @param aucinf the AUC from time 0 to infinity (typically using 
+#'   single-dose data)
+#' @param tau the dosing interval
+#' @details Note that if \code{aucinf == auctau} (as would be the 
+#'   assumption with linear kinetics), the equation becomes the same as 
+#'   the single-dose MRT.
+#' @seealso \code{\link{pk.calc.mrt}}
+#' @export
+pk.calc.mrt.md <- function(auctau, aumctau, aucinf, tau) {
+  ret <- aumctau/auctau + tau*(aucinf-auctau)/auctau
+  mask_zero <- is.na(auctau) | auctau <= 0
+  if (any(mask_zero)) {
+    ret[mask_zero] <- NA_real_
+  }
+  ret
+}
+add.interval.col("mrt.md.obs",
+                 FUN="pk.calc.mrt.md",
+                 values=c(FALSE, TRUE),
+                 desc="The mean residence time with multiple dosing and nonlinear kinetics using observed Clast",
+                 formalsmap=list(auctau="auclast", aumctau="aumclast", aucinf="aucinf.obs"),
+                 depends=c("auclast", "aumclast", "aucinf.obs"))
+PKNCA.set.summary("mrt.md.obs", business.geomean, business.geocv)
+add.interval.col("mrt.md.pred",
+                 FUN="pk.calc.mrt.md",
+                 values=c(FALSE, TRUE),
+                 desc="The mean residence time with multiple dosing and nonlinear kinetics using predicted Clast",
+                 formalsmap=list(auctau="auclast", aumctau="aumclast", aucinf="aucinf.pred"),
+                 depends=c("auclast", "aumclast", "aucinf.pred"))
+PKNCA.set.summary("mrt.md.obs", business.geomean, business.geocv)
+
 #' Calculate the terminal volume of distribution (Vz)
 #'
-#' @param cl,cl.obs,cl.pred the clearance (or apparent observed clearance)
+#' @param cl the clearance (or apparent observed clearance)
 #' @param lambda.z the elimination rate
 #' @export
 pk.calc.vz <- function(cl, lambda.z) {
@@ -439,71 +554,102 @@ pk.calc.vz <- function(cl, lambda.z) {
     stop("'cl' and 'lambda.z' must be the same length")
   cl/lambda.z
 }
-#' @describeIn pk.calc.vz Calculate Vz using observed Clast
-#' @export
-pk.calc.vz.obs <- function(cl.obs, lambda.z)
-  pk.calc.vz(cl.obs, lambda.z)
-#' @describeIn pk.calc.vz Calculate Vz using predicted Clast
-#' @export
-pk.calc.vz.pred <- function(cl.pred, lambda.z)
-  pk.calc.vz(cl.pred, lambda.z)
 ## Add the columns to the interval specification
 add.interval.col("vz.obs",
-                 FUN="pk.calc.vz.obs",
+                 FUN="pk.calc.vz",
                  values=c(FALSE, TRUE),
                  desc="The terminal volume of distribution using observed Clast",
+                 formalsmap=list(cl="cl.obs"),
                  depends=c("cl.obs", "lambda.z"))
 PKNCA.set.summary("vz.obs", business.geomean, business.geocv)
 add.interval.col("vz.pred",
-                 FUN="pk.calc.vz.pred",
+                 FUN="pk.calc.vz",
                  values=c(FALSE, TRUE),
                  desc="The terminal volume of distribution using predicted Clast",
+                 formalsmap=list(cl="cl.pred"),
                  depends=c("cl.pred", "lambda.z"))
 PKNCA.set.summary("vz.pred", business.geomean, business.geocv)
 
 #' Calculate the steady-state volume of distribution (Vss)
 #'
-#' @param cl,cl.obs,cl.pred the clearance
-#' @param mrt,mrt.obs,mrt.pred the mean residence time
+#' @param cl the clearance
+#' @param mrt the mean residence time
 #' @return the volume of distribution at steady-state
 #' @export
 pk.calc.vss <- function(cl, mrt)
   cl*mrt
-#' @describeIn pk.calc.vss Vss calculation using observed Clast
-#' @export
-pk.calc.vss.obs <- function(cl.obs, mrt.obs)
-  pk.calc.vss(cl.obs, mrt.obs)
-#' @describeIn pk.calc.vss Vss calculation using predicted Clast
-#' @export
-pk.calc.vss.pred <- function(cl.pred, mrt.pred)
-  pk.calc.vss(cl.pred, mrt.pred)
 # Add the columns to the interval specification
 add.interval.col("vss.obs",
-                 FUN="pk.calc.vss.obs",
+                 FUN="pk.calc.vss",
                  values=c(FALSE, TRUE),
                  desc="The steady-state volume of distribution using observed Clast",
+                 formalsmap=list(cl="cl.obs", mrt="mrt.obs"),
                  depends=c("cl.obs", "mrt.obs"))
 PKNCA.set.summary("vss.obs", business.geomean, business.geocv)
 add.interval.col("vss.pred",
-                 FUN="pk.calc.vss.pred",
+                 FUN="pk.calc.vss",
                  values=c(FALSE, TRUE),
                  desc="The steady-state volume of distribution using predicted Clast",
+                 formalsmap=list(cl="cl.pred", mrt="mrt.pred"),
                  depends=c("cl.pred", "mrt.pred"))
 PKNCA.set.summary("vss.pred", business.geomean, business.geocv)
+add.interval.col("vss.last",
+                 FUN="pk.calc.vss",
+                 values=c(FALSE, TRUE),
+                 desc="The steady-state volume of distribution calculating through Tlast",
+                 formalsmap=list(cl="cl.last", mrt="mrt.last"),
+                 depends=c("cl.last", "mrt.last"))
+PKNCA.set.summary("vss.last", business.geomean, business.geocv)
+add.interval.col("vss.iv.obs",
+                 FUN="pk.calc.vss",
+                 values=c(FALSE, TRUE),
+                 desc="The steady-state volume of distribution with intravenous infusion using observed Clast",
+                 formalsmap=list(cl="cl.obs", mrt="mrt.iv.obs"),
+                 depends=c("cl.obs", "mrt.iv.obs"))
+PKNCA.set.summary("vss.iv.obs", business.geomean, business.geocv)
+add.interval.col("vss.iv.pred",
+                 FUN="pk.calc.vss",
+                 values=c(FALSE, TRUE),
+                 desc="The steady-state volume of distribution with intravenous infusion using predicted Clast",
+                 formalsmap=list(cl="cl.pred", mrt="mrt.iv.pred"),
+                 depends=c("cl.pred", "mrt.iv.pred"))
+PKNCA.set.summary("vss.iv.pred", business.geomean, business.geocv)
+add.interval.col("vss.iv.last",
+                 FUN="pk.calc.vss",
+                 values=c(FALSE, TRUE),
+                 desc="The steady-state volume of distribution with intravenous infusion calculating through Tlast",
+                 formalsmap=list(cl="cl.last", mrt="mrt.iv.last"),
+                 depends=c("cl.last", "mrt.iv.last"))
+PKNCA.set.summary("vss.iv.last", business.geomean, business.geocv)
 
-#' Calculate the volume of distribution (Vd) or observed volume of distribution 
-#' (Vd/F)
+add.interval.col("vss.md.obs",
+                 FUN="pk.calc.vss",
+                 values=c(FALSE, TRUE),
+                 desc="The steady-state volume of distribution for nonlinear multiple-dose data using observed Clast",
+                 formalsmap=list(cl="cl.last", mrt="mrt.md.obs"),
+                 depends=c("cl.last", "mrt.md.obs"))
+PKNCA.set.summary("vss.md.obs", business.geomean, business.geocv)
+add.interval.col("vss.md.pred",
+                 FUN="pk.calc.vss",
+                 values=c(FALSE, TRUE),
+                 desc="The steady-state volume of distribution for nonlinear multiple-dose data using predicted Clast",
+                 formalsmap=list(cl="cl.last", mrt="mrt.md.pred"),
+                 depends=c("cl.last", "mrt.md.pred"))
+PKNCA.set.summary("vss.md.pred", business.geomean, business.geocv)
+
+#' Calculate the volume of distribution (Vd) or observed volume of
+#' distribution (Vd/F)
 #' 
 #' @param dose One or more doses given during an interval
-#' @param aucinf,aucinf.obs,aucinf.pred Area under the curve to
-#'   infinity (either predicted or observed).
+#' @param aucinf Area under the curve to infinity (either predicted or
+#'   observed).
 #' @param lambda.z Elimination rate constant
-#' @details If \code{dose} is the same length as the other inputs,
-#'   then the output will be the same length as all of the inputs; the
-#'   function assumes that you are calculating for multiple intervals
-#'   simultaneously.  If the inputs other than \code{dose} are scalars
-#'   and \code{dose} is a vector, then the function assumes multiple
-#'   doses were given in a single interval, and the sum of the
+#' @details If \code{dose} is the same length as the other inputs, then
+#'   the output will be the same length as all of the inputs; the 
+#'   function assumes that you are calculating for multiple intervals 
+#'   simultaneously.  If the inputs other than \code{dose} are scalars 
+#'   and \code{dose} is a vector, then the function assumes multiple 
+#'   doses were given in a single interval, and the sum of the 
 #'   \code{dose}s will be used for the calculation.
 #' @return The observed volume of distribution
 #' @export
@@ -512,27 +658,28 @@ pk.calc.vd <- function(dose, aucinf, lambda.z) {
       length(lambda.z) == 1) {
     dose <- sum(dose)
   }
-  dose/(aucinf * lambda.z)
+  ret <- dose/(aucinf * lambda.z)
+  mask_zero <-
+    is.na(aucinf) | aucinf <= 0 |
+    is.na(lambda.z) | lambda.z <= 0
+  if (any(mask_zero)) {
+    ret[mask_zero] <- NA_real_
+  }
+  ret
 }
-#' @describeIn pk.calc.vd Compute the Vd with observed Clast
-#' @export
-pk.calc.vd.obs <- function(dose, aucinf.obs, lambda.z)
-  pk.calc.vd(dose, aucinf.obs, lambda.z)
-#' @describeIn pk.calc.vd Compute the Vd with predicted Clast
-#' @export
-pk.calc.vd.pred <- function(dose, aucinf.pred, lambda.z)
-  pk.calc.vd(dose, aucinf.pred, lambda.z)
 ## Add the columns to the interval specification
 add.interval.col("vd.obs",
-                 FUN="pk.calc.vd.obs",
+                 FUN="pk.calc.vd",
                  values=c(FALSE, TRUE),
                  desc="Apparent observed volume of distribution calculated with observed Clast",
+                 formalsmap=list(aucinf="aucinf.obs"),
                  depends=c("aucinf.obs", "lambda.z"))
 PKNCA.set.summary("vd.obs", business.geomean, business.geocv)
 add.interval.col("vd.pred",
-                 FUN="pk.calc.vd.pred",
+                 FUN="pk.calc.vd",
                  values=c(FALSE, TRUE),
                  desc="Apparent observed volume of distribution calculated with predicted Clast",
+                 formalsmap=list(aucinf="aucinf.pred"),
                  depends=c("aucinf.pred", "lambda.z"))
 PKNCA.set.summary("vd.pred", business.geomean, business.geocv)
 
@@ -543,8 +690,14 @@ PKNCA.set.summary("vd.pred", business.geomean, business.geocv)
 #' @param end The ending time of the interval
 #' @return The Cav (average concentration during the interval)
 #' @export
-pk.calc.cav <- function(auclast, start, end)
-  auclast/(end-start)
+pk.calc.cav <- function(auclast, start, end) {
+  ret <- auclast/(end-start)
+  mask_zero <- is.na(end) | is.na(start) | end == start
+  if (any(mask_zero)) {
+    ret[mask_zero] <- NA_real_
+  }
+  ret
+}
 add.interval.col("cav",
                  FUN="pk.calc.cav",
                  values=c(FALSE, TRUE),
@@ -566,7 +719,7 @@ pk.calc.ctrough <- function(conc, time, start) {
   if (sum(mask.start) == 1) {
     conc[mask.start]
   } else if (sum(mask.start) == 0) {
-    NA
+    NA_real_
   } else {
     # This should be impossible as check.conc.time should catch
     # duplicates.
@@ -588,7 +741,7 @@ PKNCA.set.summary("ctrough", business.geomean, business.geocv)
 #' @export
 pk.calc.ptr <- function(cmax, cmin) {
   ret <- cmax/cmin
-  ret[cmin %in% 0] <- NA
+  ret[cmin %in% 0] <- NA_real_
   ret
 }
 add.interval.col("ptr",
@@ -612,7 +765,7 @@ pk.calc.tlag <- function(conc, time) {
   if (any(mask.increase)) {
     time[mask.increase][1]
   } else {
-    NA
+    NA_real_
   }
 }
 add.interval.col("tlag",
@@ -630,7 +783,12 @@ PKNCA.set.summary("tlag", business.median, business.range)
 #' @return The degree of fluctuation around the average concentration.
 #' @export
 pk.calc.deg.fluc <- function(cmax, cmin, cav) {
-  100*(cmax - cmin)/cav
+  ret <- 100*(cmax - cmin)/cav
+  mask_zero <- is.na(cav) | cav == 0
+  if (any(mask_zero)) {
+    ret[mask_zero] <- NA_real_
+  }
+  ret
 }
 add.interval.col("deg.fluc",
                  FUN="pk.calc.deg.fluc",
@@ -657,3 +815,30 @@ add.interval.col("swing",
                  desc="Swing relative to Cmin",
                  depends=c("cmax", "cmin"))
 PKNCA.set.summary("swing", business.mean, business.sd)
+
+#' Determine the concentration at the end of infusion
+#' 
+#' @param conc Concentration measured
+#' @param time Time of concentration measurement
+#' @param duration.dose The duration for the dosing administration 
+#'   (typically from IV infusion)
+#' @param check Run \code{\link{check.conc.time}}?
+#' @return The concentration at the end of the infusion, \code{NA} if
+#'   duration.dose is \code{NA}, or \code{NA} if all \code{time != duration.dose}
+#' @export
+pk.calc.ceoi <- function(conc, time, duration.dose=NA, check=TRUE) {
+  if (check)
+    check.conc.time(conc=conc, time=time)
+  if (is.na(duration.dose)) {
+    NA_real_
+  } else if (all(time != duration.dose)) {
+    NA_real_
+  } else {
+    conc[time == duration.dose][1]
+  }
+}
+add.interval.col("ceoi",
+                 FUN="pk.calc.ceoi",
+                 desc="Concentration at the end of infusion",
+                 depends=c())
+PKNCA.set.summary("ceoi", business.geomean, business.geocv)
